/*
 * SPI Programmer Example for FT2232 devices using libftdi
 * (c)2010 Tomek Cedro (http://www.tomek.cedro.info)
 *
 */


#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <ftdi.h>

int cable_vid=0x0403; //this is ftdi vid, most common to match
//int cable_pid=0xbbe2; //Kristech KT-LINK
int cable_pid=0xcff8; //Amontec JTAGKey
//Olimex ARM-USB-OCD
//int cable_vid=0x15ba;
//int cable_pid=0x003;

struct ftdi_context *ftdic;
struct ftdi_device_list devlist;
unsigned char buf[256];

int list_ftdi_devices(struct ftdi_context *ftdic, struct ftdi_device_list *devlist, int vid);
int ftdiInit(int vid, int pid);
int ftdiDeInit();
int checkForErrors(int result);
int testBitbangCount();
int testBitbangShift();
int testMpsse();
void eject();
void bufferFill(unsigned char val);


//////////////////////////
// MAIN
//////////////////////////

int main(){
 int ret, ret1, ret2, ret3;

 signal(SIGINT, eject); 
 signal(SIGSEGV, eject); 
 signal(SIGINT, eject); 

 ftdiInit(cable_vid, cable_pid);

// testBitbangShift();

 testMpsse();
 return 0;
 
 ftdi_disable_bitbang(ftdic);

 return ftdiDeInit(&ftdic);
}


int list_ftdi_devices(struct ftdi_context *ftdic, struct ftdi_device_list *devlist, int vid){
 //DOES NOT WORK YET ;P
 return -1;
 if (vid==0){
  fprintf(stderr, "You must supply VID!");
  return -1;
 } 
 int pid, ret, found;
 printf("Searching for PID: %x", pid);
 for (pid=0; pid<0xfff; pid++){
  if (!pid%10) printf("...%x", pid);
//  if ( (ret=ftdi_usb_find_all(ftdic, devlist, vid, pid)) < 0){
//   fprintf(stderr, "ftdi_usb_find_all failed: %d (%s)\n", ret, ftdi_get_error_string(&ftdic));
//   return EXIT_FAILURE;
//  } else found+=ret; 
 }
 printf("Found %d devices\n", found);
 return found;
}

int ftdiInit(int vid, int pid){
 int ret;
 ftdic = (struct ftdi_context *) calloc(1, sizeof(struct ftdi_context));
 if (ftdi_init(ftdic) < 0){
  fprintf(stderr, "ERROR: ftdi_init failed!\n");
  exit(EXIT_FAILURE);
 }
 if ((ret = ftdi_usb_open(ftdic, cable_vid, cable_pid)) < 0){
  fprintf(stderr, "ERROR %d: unable to open ftdi device VID:0x%04X PID:0x%04X (%s)\n", ret, vid, pid, ftdi_get_error_string(ftdic));
  ftdi_deinit(ftdic);
  exit(EXIT_FAILURE);
 }
 unsigned int chipid;
 ftdi_read_chipid(ftdic, &chipid);
 printf("FTDI chipid: %X\n", chipid);
 return 0;
}

int ftdiDeInit(){
 int ret;
 unsigned char portoff=0xFF;

 //set all port A lines as inputs (to a high state for target separation)
 ftdi_set_bitmode(ftdic, 0, BITMODE_BITBANG);
 //perform shutdown - close usb connection, destroy/free context data
 if ((ret = ftdi_usb_close(ftdic)) < 0){
  fprintf(stderr, "ERROR %d: Unable to close ftdi device (%s)\n", ret, ftdi_get_error_string(ftdic));
  ftdi_deinit(ftdic);
  return EXIT_FAILURE;
 }
 ftdi_deinit(ftdic);
 free(ftdic);
 return EXIT_SUCCESS;
}

int checkForErrors(int result){
 if (result < 0){
  fprintf(stderr, "ERROR: %s\n", ftdi_get_error_string(ftdic));
  exit(result);
 }
}

int testBitbangCount(){
 int ret1, ret2, ret3, x;

 buf[0]=0;
 buf[1]=0xFF;

 //Each interface must have its own context associated!
 struct ftdi_context ftdica=*ftdic, ftdicb=*ftdic;

 ret1=ftdi_set_interface(&ftdica, INTERFACE_A);
 ret2=ftdi_set_bitmode(&ftdica, 0xFF, BITMODE_BITBANG);
 ret3=ftdi_write_data(&ftdica, &buf[0], 1);
 checkForErrors(ret1|ret2|ret3);

 ret1=ftdi_set_interface(&ftdicb, INTERFACE_B);
 ret2=ftdi_set_bitmode(&ftdicb, 0xFF, BITMODE_BITBANG);
 ret3=ftdi_write_data(&ftdicb, &buf[0], 1);
 checkForErrors(ret1|ret2|ret3);

 for (x=0;x<1000;x++){
  ftdi_write_data(&ftdica, &buf[0], 1);
  ftdi_write_data(&ftdicb, &buf[1], 1);
  printf("(A:%02X,B:%02X) ", buf[0], buf[1]);
  printf(x%8==0?"\n":"");
  buf[0]++;
  buf[1]=buf[0]*2;
  usleep(50000);
 }
 return 0;
}

int testBitbangShift(){
 int ret1, ret2, ret3, x;

 buf[0]=1;
 buf[1]=0xFF;

 //Each interface must have its own context associated!
 struct ftdi_context ftdica=*ftdic, ftdicb=*ftdic;

 ret1=ftdi_set_interface(&ftdica, INTERFACE_A);
 ret2=ftdi_set_bitmode(&ftdica, 0xFF, BITMODE_BITBANG);
 ret3=ftdi_write_data(&ftdica, &buf[0], 1);
 checkForErrors(ret1|ret2|ret3);

 ret1=ftdi_set_interface(&ftdicb, INTERFACE_B);
 ret2=ftdi_set_bitmode(&ftdicb, 0xFF, BITMODE_BITBANG);
 ret3=ftdi_write_data(&ftdicb, &buf[0], 1);
 checkForErrors(ret1|ret2|ret3);

 for (x=0;x<=8;x++){
  buf[1]=buf[0]<<x;
  ftdi_write_data(&ftdica, &buf[1], 1);
  ftdi_write_data(&ftdicb, &buf[1], 1);
  printf("Output on port A and B: 0x%02X. Press Enter to next step. \n", buf[1]);
  getchar();
 }
 return 0;
}






int testMpsse(){
 int ret1, ret2, ret3, x;
 unsigned char portHdir, portLdir;
 struct ftdi_context ftdicma=*ftdic; //we want to work on a local context copy assigned to port A

 
 portLdir = 0 | (1<<0) | (1<<1) | (1<<3) | (1<<4); //this is setup for amontec cable
 ret1=ftdi_set_interface(&ftdicma, INTERFACE_A);
 ret2=ftdi_set_bitmode(&ftdicma, portLdir, BITMODE_MPSSE);
 checkForErrors(ret1|ret2);

 //setup port A direction (1 is output)
 portHdir=0;
 buf[0]=0x80;     //Set Data bits LowByte
 buf[1]=0;        //Value
 buf[2]=portLdir; //Direction
 ret1=ftdi_write_data(&ftdicma, buf, 3); 
 checkForErrors(ret1);
 buf[0]=0x82;
 buf[1]=0;
 buf[2]=portHdir;
 ret1=ftdi_write_data(&ftdicma, buf, 3);
 checkForErrors(ret1);
 
 //enable loopback
 buf[0]=0x84;
 ftdi_write_data(&ftdicma, buf, 0);

 while (getchar()!=' ') {
  buf[0]=0x38;
  buf[1]=4;
  buf[2]='T';
  buf[3]='E';
  buf[4]='S';
  buf[5]='T';
  ret1=ftdi_write_data(&ftdicma, buf, 6);
  printf("TX: ");  for (x=0;x<=10;x++){printf("%02X ", buf[x]);}; printf("\n");
  bufferFill(0xFF);
  ret2=ftdi_read_data(&ftdicma, buf, 10);
  printf("RX: "); for (x=0;x<=10;x++){printf("%02X ", buf[x]);}; printf("\n");
  checkForErrors(ret1|ret2);
  bufferFill(0xFF);
 }

 //disable loopback
 buf[0]=0x85;
 ftdi_write_data(ftdic, buf, 0);

 return 0;
}

void eject(void){
 printf("\nEjecting...\n");
 exit(ftdiDeInit());
}

void bufferFill(unsigned char val){
 int x;
 for (x=0;x<=sizeof(buf)-1;x++) buf[x]=val;
 return;
}
